# ModelTranscribers

The gem can help you copy any value of any attributes from a model (**progenitor**) to another (**transcript**) in real time.

A new record of a transcript will also be created just right after the new record of the progenitor has been created.

If any value of the progenitor's attributes that you have mapped in advance in the progenitor is changed, the value of the transcript's corresponding attributes will be updated automatically.

# Install

Add this line to your applocation's Gemfile:
```
gem 'model_transcribers'
```
And then execute:
```
$ bundle install  
```
We also need to add a foreign_key called progenitor_id to the transcript table（For example, if your progenitor model called ***Student*** and its transcript called ***User***):
```
$ rails g migration AddProgenitorIdToUser progenitor_id:integer
```

## Usage

In the progenitor model:
```ruby
# Student is progenitor.
class Student < ActiveRecord::Base

  include ModelTranscribers

  # User is transcript.
  sync transcript: User do
    copy_attr from: :id_number, to: :account
    copy_attr from: :phone_number, to: :phone_number
    copy_attr from: :status, to: :status, 
        by: lambda {
          case self.status
          when :enrolled then :active
          when :suspension then :inactive
        }
    assign_attr to: :role, by: -> { 'Student' }
    assign_attr to: :name, by: -> { "#{first_name} #{last_name}" }
  end
end
```

Copy a value between attributes with the same name:
```ruby
  copy_attr from: :phone_number, to: :phone_number
```

Copy a value from an attributes to another:
```ruby
  copy_attr from: :id_number, to: :account
```

Copy a value by extra work:
```ruby
  copy_attr from: :status, to: :status, 
            by: lambda {
              case self.status
              when :enrolled then :active
              when :suspension then :inactive
            }
```

Assign a value directly:
```ruby
assign_attr to: :name, by: -> { "#{first_name} #{last_name}" }
```

Use the association between a progenitor and its transcript:
```ruby
student = Student.take
student.transcript

user = User.find_by(transcript: student)
user.progenitor
```

When you create a new record of progenitor, the corresponding transcript will also be created automatically:
```ruby
student = Student.create(first_name: 'Joey', last_name: 'Chung', 
                         phone_number: '+88628825252',
                         status: :enrolled)
user = student.transcript

puts user.name         # => "Joey Chung"
puts user.phone_number # => "+88628825252"
puts user.satus        # => ":active"
```


## Notice
If you mapped the same attribute in the **from:** twice, only the last mapping wil be effective:
```ruby
copy_attr frome: :name, to: :first_name, by: -> { name.split(' ').first }
copy_attr frome: :name, to: :last_name, by: -> { name.split(' ').last } # Only this one effective.
```

## Contributing

1. Fork it ( http://github.com/hugtrueme/model_transcribers/fork )
2. Create your feature branch (`git checkout -b my-new-feature`)
3. Commit your changes (`git commit -am 'Add some feature'`)
4. Push to the branch (`git push origin my-new-feature`)
5. Create new Pull Request
